# -*- coding: utf-8 -*-

import logging
import os
import random
import string

import imageio
import datetime
import json
import subprocess

from multiprocessing import Pool
from PIL import Image, ImageDraw, ImageFont
from pyrest.assets import assetsDir

from pyrest.utils.logger.logger import Logger


class TimelapseFactory:
    @staticmethod
    def factory(type):
        if type == "GIF":
            return GifTimeLapse()
        else:
            raise Exception("Cannot create timelapse of type " + type)


class Timelapse(object):

    def __init__(self):
        self.images = list()
        self.tmp_dir = "/tmp/"
        self.logging = Logger('timelapse.log')


    def addWMSImage(self, text, url, application, language):
        data = dict()
        data['text'] = text
        data['url'] = url
        data['application'] = application
        data['language'] = language
        self.images.append(data)

    def fetchImages(self):
        p = Pool(5)
        self.logging.debug("Fetching images %s" % ','.join(json.dumps(image) for image in self.images))
        imageList = list()
        imageIt = p.imap(fetchImage, self.images)
        for image in self.images:
            imageList.append(imageIt.next(timeout=30000))
        p.close()
        return imageList

def fetchImage(image):
    #print('loading image %s' % image["url"])
    frame = Image.fromarray(imageio.imread(uri=image["url"], format="PNG" ))
    frame = addLogo(image=frame, application=image['application'])
    frame = addCopyright(image=frame, text=getCopyrightTxt(application=image['application'], language=image['language']).decode('utf-8') % image['text'].split("/")[2])
    frame = addText(image=frame, text=image['text'])
    return (image["text"], frame)

def addLogo(image, application):
    logoDir = '%s/logos/%s.png' % (assetsDir , application)
    logo = Image.open(logoDir).convert("RGBA")

    # resize logo
    wsize = int(min(image.size[0], image.size[1]) * 0.1)
    wpercent = (wsize / float(logo.size[0]))
    hsize = int((float(logo.size[1]) * float(wpercent)))

    simage = logo.resize((wsize, hsize), resample=Image.LANCZOS)
    mbox = tuple([0, 0, image.size[0] - 1, image.size[1] - 1])
    sbox = simage.getbbox()

    # right bottom corner
    box = (mbox[2] - sbox[2] - 10, mbox[3] - sbox[3] -10)
    image.paste(simage, box, simage)
    return image


def addText(image, text):

    font = ImageFont.truetype("%s/fonts/OpenSans-Bold.ttf" % assetsDir, 15)

    text_size = font.getsize(text)
    background_size = (text_size[0] + 10, text_size[1] + 13)

    textComponent = Image.new('RGBA', image.size, (0, 0, 0, 0))
    draw = ImageDraw.Draw(textComponent)
    draw.rectangle(((0, 0), (background_size[0], background_size[1])), fill=(255, 255, 255, 204))
    draw.text((5, 5), text, (0, 0, 0), font=font)

    # put button on source image in position (0, 0)
    image.paste(textComponent, (0,0), textComponent)
    return image

def addCopyright(image, text):

    font = ImageFont.truetype("%s/fonts/OpenSans-Bold.ttf" % assetsDir, 12)

    text_size = font.getsize(text)

    textComponent = Image.new('RGBA', text_size, (0, 0, 0, 0))
    draw = ImageDraw.Draw(textComponent)
    draw.text((0,0), text, (255, 255, 255), font=font)
    rotatedText = textComponent.rotate(-90.0, expand=1)

    image.paste(rotatedText, (image.size[0] - rotatedText.size[0],5), rotatedText)
    return image

def getCopyrightTxt(application, language="en"):
    crTxt = dict()
    crTxt["terrascope"] = dict()
    crTxt["terrascope"]["en"] = "Contains modified Copernicus Sentinel data %s"
    crTxt["terrascope"]["nl"] = "Bevat bewerkte Copernicus Sentinel data  %s"
    crTxt["terrascope"]["fr"] = "Contient des donn\xc3\xa9es Copernicus Sentinel modifi\xc3\xa9es %s"
    crTxt["probav"] = dict()
    crTxt["probav"]["en"] = "PROBA-V \xc2\xa9 %s ESA/BELSPO"
    crTxt["probav"]["nl"] = "PROBA-V \xc2\xa9 %s ESA/BELSPO"
    crTxt["probav"]["fr"] = "PROBA-V \xc2\xa9 %s ESA/BELSPO"
    return crTxt[application][language]

class GifTimeLapse(Timelapse):

    def create(self, application, language, delay=1, loop=0):
        save_dir = os.path.join(self.tmp_dir, ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(10)))
        self.logging.debug("Creating temporaray directory at %s " % (save_dir))
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        gif_name = save_dir + "/pygif.gif"
        self.logging.debug("Fetching %s images" % (str(len(self.images))))
        counter = 0
        images = sorted(self.fetchImages(), key=lambda x:datetime.datetime.strptime(x[0], "%d/%m/%Y"))
        imagefiles = list()
        for frame in images:
            imagefn = "%s/image_%d.png" % (save_dir, counter)
            frame[1].save(imagefn)
            counter+=1
            imagefiles.append(imagefn)
        self.logging.debug("Creating gif image with delay of %s and looping set to %s" % (str(delay), str(loop)))
        command = "convert -delay %d -loop %d %s %s" % (delay / 10, loop, ' '.join([fn for fn in imagefiles]), gif_name)
        self.logging.debug(command)
        subprocess.call(command, shell=True)
        return gif_name
